@@ -1533,7 +1533,7 @@ pltcl_elog(ClientData cdata, Tcl_Interp *interp,
 	if (level == ERROR)
 	{
 		/*
-		 * We just pass the error back to Tcl.	If it's not caught, it'll
+		 * We just pass the error back to Tcl.  If it's not caught, it'll
 		 * eventually get converted to a PG error when we reach the call
 		 * handler.
 		 */
@@ -2161,9 +2161,9 @@ pltcl_SPI_execute_plan(ClientData cdata, Tcl_Interp *interp,
 	int			j;
 	Tcl_HashEntry *hashent;
 	pltcl_query_desc *qdesc;
-	const char *volatile nulls = NULL;
-	CONST84 char *volatile arrayname = NULL;
-	CONST84 char *volatile loop_body = NULL;
+	const char *nulls = NULL;
+	CONST84 char *arrayname = NULL;
+	CONST84 char *loop_body = NULL;
 	int			count = 0;
 	int			callnargs;
 	CONST84 char **callargs = NULL;
@@ -2293,6 +2293,8 @@ pltcl_SPI_execute_plan(ClientData cdata, Tcl_Interp *interp,
 	if (i != argc)
 	{
 		Tcl_SetResult(interp, usage, TCL_STATIC);
+		if (callargs)
+			ckfree((char *) callargs);
 		return TCL_ERROR;
 	}
 
@@ -2331,10 +2333,6 @@ pltcl_SPI_execute_plan(ClientData cdata, Tcl_Interp *interp,
 			}
 		}
 
-		if (callargs)
-			ckfree((char *) callargs);
-		callargs = NULL;
-
 		/************************************************************
 		 * Execute the plan
 		 ************************************************************/
@@ -2361,6 +2359,9 @@ pltcl_SPI_execute_plan(ClientData cdata, Tcl_Interp *interp,
 	}
 	PG_END_TRY();
 
+	if (callargs)
+		ckfree((char *) callargs);
+
 	return my_rc;
 }
 
